<?php

/**
*
* Plugin Name: TTFGen for Wordpress
* Plugin URI: http://www.ttfgen.com/wordpress
* Description: TTFGen will allow you to upload your own custom fonts and embed them directly into your wordpress page.
* Version: 0.9.4
* Author: Mike Dabbs, Vinny Troia
* Author URI: http://www.ttfgen.com/
* 
* 
*/  

define ( 'TTF_OPTIONS','ttfgen_options' );
define ( 'TTF_FONTS', 'ttfgen_fonts' ); 
define ( 'TTF_CSS', 'ttfgen_css' ); 
define ( 'TTF_VERSION','0.9.4' );

$nonce_base = 'wp-ttfgen-nonce'; // nonce for security of links/forms, try to prevent "CSRF"

$fontList = array(); 
$tgStyles = array(); 

		
// Folder definitions as constants	
define ( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' );	// ex: /var/www/ttfgen/httpdocs/wp-content 	
define ( 'WP_CONTENT_URL', get_option('siteurl') . '/wp-content' );	// ex: http://www.ttfgen.com/wp-content 	 	
define ( 'WP_PLUGIN_URL', WP_CONTENT_URL . '/plugins' );	// ex: http://www.ttfgen.com/wp-content/plugins
define ( 'WP_PLUGIN_DIR', WP_CONTENT_DIR . '/plugins');	// ex: /var/www/ttfgen/httpdocs/wp-content/plugins/
define ( 'WP_TTFGEN_PATH', WP_PLUGIN_DIR . '/' . basename( dirname ( __FILE__ ) ) . '/' );	// ex: /var/www/ttfgen/httpdocs/wp-content/plugins/wp-ttfgen/ 	
define ( 'TTF_PATH_SHORT', '/wp-content/plugins/wp-ttfgen/');	// ex: /wp-content/plugins/wp-ttfgen/ 	
define ( 'WP_TTFGEN_URL', WP_PLUGIN_URL . '/' . basename( dirname ( __FILE__ ) ) . '/' );	// ex: http://www.ttfgen.com/wp-content/plugins/wp-ttfgen/ 
define ( 'WP_TTFGEN_BASENAME', plugin_basename( __FILE__ ) );


// init table names  
global $table_prefix;
$wp_ttf_fonts = $table_prefix."ttfgen_fonts"; 
$wp_ttf_css = $table_prefix."ttfgen_css";


define ( 'WP_DB_FONTS', $table_prefix."ttfgen_fonts" ); 
define ( 'WP_DB_CSS', $table_prefix."ttfgen_css" ); 


// Set plugin admin menu 
add_action( 'admin_menu', array(&$this, 'adminMenu') );

// multi language support
load_plugin_textdomain('wp-ttfgen', WP_TTFGEN_PATH.'languages', 'languages/');

 
//add TTFGen to settings page
function ttfAdminMenu() 
{
	if (function_exists('add_options_page')) {
		add_options_page('TTFGen - Embed Custom Fonts', 'TTFGen', 9, basename(__FILE__), 'ttfOptionsPage');
	}
}

$ttfOptions = get_option(TTF_OPTIONS);

// Options do not exist or have not yet been loaded so we define standard options...
// if(!is_array($ttfOptions)) 

if ($ttfOptions['version'] != TTF_VERSION) 
{
	$ttfOptions = array( 
		'version' => TTF_VERSION,
		'ttfPath' => WP_TTFGEN_PATH,
		'jqueryPath' => WP_TTFGEN_PATH.'scripts/jquery.js',
		'useJquery' => true,
		'useMeta' => true,
		'fontFolder' => WP_TTFGEN_PATH.'fonts/',
		'fontCache' => WP_TTFGEN_PATH.'fontcache/',
		'loadLocation' => 'footer',
		'version' => TTF_VERSION	
	);
	
	//add or update the db
	update_option(TTF_OPTIONS, $ttfOptions);
} 
	 
//reload
$ttfOptions = get_option(TTF_OPTIONS);

// create font folder if it doesn't exist
if (!file_exists($ttfOptions['fontFolder'])) {
	mkdir($ttfOptions['fontFolder'], 0760); 
} 

// create fontCache folder
if (!file_exists($ttfOptions['fontCache'])) {
	mkdir($ttfOptions['fontCache'], 0764); 
} 
				
// init plugin
init_plugin();
		  


/**
 * INIT PLUGIN 
 * Creates database table (if not found) and checks for any post data. 
 * If post found, redirect accordingly. 
 * 
 */

function init_plugin() 
{	
   global $wpdb, $ttfOptions;
   
	$fontDb = "CREATE TABLE IF NOT EXISTS ".WP_DB_FONTS." (				
			`id`        INT(10) UNSIGNED NOT NULL AUTO_INCREMENT, 
			`name`   	LONGTEXT  NOT NULL ,
			`font`  	LONGTEXT  NOT NULL ,
			PRIMARY KEY ( `id` ));";
	$result = $wpdb->query($fontDb);
	 		   		   
   	$cssDb = "CREATE TABLE IF NOT EXISTS ".WP_DB_CSS." (				
           `id` int(10) UNSIGNED NOT NULL AUTO_INCREMENT,                                                                 
           `css_type` varchar(100) NOT NULL ,                                                                          
           `css_name` varchar(200) NOT NULL ,                                                                         
           `css_font_id` int(10) NOT NULL,  
			PRIMARY KEY ( `id` ));";
	$cssResult = $wpdb->query($cssDb);
 

	// If page is loading from POST, determine where it is going 			 
	if ($_POST)
	{		
		switch ($_GET['action'])
		{
			case "add":
				addFont();
				break; 
			case "deleteFont":
				deleteFont(); 
				break; 
			case "deleteCSS":
				deleteCSS(); 
				break; 
			case "assignCSS":
				assignCSS(); 
				break; 
		}	
	}
	
}
					

// ************* Add a New Font

function addFont() 
{
	global $wpdb, $ttfOptions;	
	

	// Valid file types for upload.
	$allowedExtensions = array("ttf"); 

	if (empty( $_POST['fontFileName'] ) && empty($_FILES['uploadedFile']['tmp_name'])) 
		$_SESSION['msg'] .= '<div class="error">'._e('Required field: <strong>File URL</strong> omitted',"wp-ttfgen").'</div>';	

	if (empty( $_POST['fontName'] )) 
		$_SESSION['msg'] .= '<div class="error">'._e('Required field: <strong>Title</strong> omitted',"wp-ttfgen").'</div>';	

	$fontName = trim($_POST['fontName']); 

	$file = basename($_FILES['uploadedFile']['name']);
	$fileExt = strtolower(substr($file, strripos($file, '.'))); // file extension
	$fileBase = strtolower(substr($file, 0, strripos($file, '.'))); // file name
	
	// strip spaces from file name
	$fileBase = str_replace (" ", "_", $fileBase);

	if ($fileExt == ".ttf") { 
		$fileBase = trim($fileBase);
		$fileName = $fileBase.".ttf"; 
		$target_path = $ttfOptions['fontFolder'] . $fileName; 
		
		// Copy uploaded file.
		if (move_uploaded_file($_FILES['uploadedFile']['tmp_name'], $target_path)) {
			chmod ($target_path, 0644);	 

			// once successful, add to database												
			$query = "INSERT INTO ".WP_DB_FONTS." (name, font) VALUES ('".$fontName."', '".$fileName."')";
			$result = $wpdb->query($query);
			
			// display success message
			$_SESSION['msg'] = "The file ". basename( $fontName ). " has been uploaded";
			
		} else {
			$_SESSION['msg'] = "File upload failed.";
		}

	} else { 
		$_SESSION['msg'] = "Font extension invalid.";
	}

	refresh(); 
}	


// ************* Assign Font to CSS 

function assignCSS()
{
	global $wpdb, $ttfOptions;

	// get post data
	$cssType = $_POST['cssType'];
	$fontName = $_POST['fontName'];
	$cssName = $_POST['cssName'];
	
	// find the font ID 
	$fontID = $wpdb->get_var("SELECT id FROM ".WP_DB_FONTS." WHERE name = \"".$fontName."\"");
	
	// insert the new data to the db
	$query = "INSERT INTO ".WP_DB_CSS." (css_name, css_type, css_font_id) VALUES ('".$cssName."', '".$cssType."', '".$fontID."')";
	
	if ($wpdb->query($query)) 
		$_SESSION['msg'] = "CSS assignment successful.";
	else 
		$_SESSION['msg'] = "CSS assignment failed. Please try again.";
	
	refresh(); 
}

function refresh() 
{
	//used to clean the url and refresh the page
	$deleteGoTo = "?page=ttfgen.php";
	header ("Location: $deleteGoTo"); 
}


// ************* Delete a Font

function deleteFont() 
{
	global $wpdb, $ttfOptions;
	$fontID = $_POST['fontID'];
	
	// get the font file name
	$fileName = $wpdb->get_var("SELECT font FROM ".WP_DB_FONTS." WHERE id=\"".$fontID."\"");
	
	// remove the font file
	$target = $ttfOptions['fontFolder'] . $fileName;
	unlink($target);
	
	// delete the CSS associated with the font. 
	$cssQ = "DELETE FROM ".WP_DB_CSS." WHERE css_font_id=\"".$fontID."\""; 
 	$wpdb->query($cssQ);

	// remove the font entry from the database
	$fontQ = "DELETE FROM ".WP_DB_FONTS." WHERE id=\"".$fontID."\""; 
	$wpdb->query($fontQ);	
	
	refresh(); 
}

// ************* Delete a CSS Assignment

function deleteCSS() 
{
	
	global $wpdb, $ttfOptions; 
	$cssID = $_POST['cssID']; 

	// remove the font entry from the database
	$fontQ = "DELETE FROM ".WP_DB_CSS." WHERE id=\"".$cssID."\""; 
	$wpdb->query($fontQ);
	
	refresh(); 
}

function getFonts() 
{
   	global $wpdb, $ttfOptions;
	$result = $wpdb->get_results("SELECT * FROM ".WP_DB_FONTS);
	return $result; 
}


function getCSS() 
{
   	global $wpdb, $ttfOptions;
	$result = $wpdb->get_results("SELECT * FROM ".WP_DB_CSS);
	return $result; 	
}

function getFontName($fontID) 
{
   	global $wpdb, $ttfOptions;
	$fontName = $wpdb->get_var("SELECT font FROM ".WP_DB_FONTS." WHERE id=\"".$fontID."\"");
	return $fontName; 	
}

/**
 * TTFGen Admin Options Page
 */
	
function ttfOptionsPage() 
{
	global $ttfOptions, $wpdb;	
	
	// break PHP to start javascript and HTML	
	?>
	
	<script type="text/javascript">

		function confirmDelete(delString) { 
			confirm.value = confirm(delString); 
		 	return confirm.value;  
		} 
					
	</script>
	 
	<style type="text/css">
		
		h3 {
			color: #21759B; 
			font-size: 15px;  
			margin: 0px; 
			padding-top: 5px; 
			padding-bottom: 10px; 
		}
		
		.rowOdd { 
			background-color:#e9e9e9; 
		}
		
		#fontTable, #cssListTable  { 
			width:100%; 
		}	
		
		#fontTable, #cssAddTable, #cssListTable  {
			text-align:left;
			padding-bottom: 10px; 
		}
		
		#fontTable th, #cssListTable th {	
			padding: 5px; 
			border-bottom: 1px solid #ccc; 
			font-size: 13px;
			color: #555; 
			font-weight: bold;  
						
		}
		
		#fontTable td, #cssListTable td { 
			padding: 5px; 
			border-bottom: 1px solid #CCC;  
		}
		
		.sectionWrapper {
			padding: 10px; 
			width: 80%; 			
		}
		.colorWrapper {
			background-color: #f0f0f0; 
			padding: 10px; 
			width: 80%; 
			border-bottom: 1px solid #ccc; 
			border-top: 1px solid #ccc;   
		
		}
		
	</style>
	
	 <div class="wrap">
	 	<h2><?php _e('TTFGen Options','ttfText'); ?></h2>
			<?php wp_nonce_field($nonce_base) ?>
		<p><?php _e("If you find this plugin useful, please consider making a PayPal donation of $2", 'wp_ttfgen'); ?>, </a></p>
	
		<strong><?php _e($_SESSION['msg'], 'wp-ttfgen'); ?></strong>
	
		<div class="colorWrapper">
			<h3>General Settings</h3>
			<form style="padding-bottom: 10px">
				<label style="font-weight:bold;"><?php _e("Where to Load TTFGen:",'wp-ttfgen'); ?> </label>
				
				<input type="radio" name="loadLocation" value="header" <?php if ($ttfOptions['loadLocation'] == "header") echo 'CHECKED'; ?>><?php _e("Header",'wp-ttfgen'); ?>
				 
				<input type="radio" name="loadLocation" value="footer" <?php if ($ttfOptions['loadLocation'] = "footer") echo 'CHECKED'; ?>> <?php _e("Footer",'wp-ttfgen'); ?>
				 
				<p><INPUT TYPE="checkbox" NAME="useJquery" width="250" id="useJquery"   
						<?php if ($ttfOptions['useJquery']) echo " checked"; ?>	> 
						<?php _e("Use included jQuery file (Do not uncheck unless you are manually loading jQuery elsewhere)",'wp-ttfgen'); ?> 	
				</p>
				<p>
				<INPUT TYPE="checkbox" NAME="useMeta" width="250" id="useMeta"   
						<?php if ($ttfOptions['useMeta']) echo " checked"; ?>	> 
						<?php _e("Use included jQuery metadata plugin (Do not uncheck unless you are manually loading the metadata plugin elsewhere)",'wp-ttfgen'); ?> 					
				</p>
				<strong><em>(settings update disabled until next version)</em></strong>
			</form>			
		</div>

		
		<!-- Display the Font List -->
		
		<div class="sectionWrapper">
			<h3>Available Fonts</h3>
			
			<?php  
				$fontList = getFonts(); 
				if ($fontList == null) 
				{ 
					_e("You have not uploaded any fonts.", 'wp-ttfgen'); 
				} 
				else 
				{
					// start the table
					echo '<table id="fontTable" border="0" cellpadding="0" cellspacing="0">
							<tr>
								<th>Name</th>
								<th>Font</th>
								<th>Action</th>
							</tr> 
						'; 
						
					for ($i=0; $i < count($fontList); $i++) 
					{
						$curID = $fontList[$i]->id; 
						$curName = $fontList[$i]->name; 
						$curFont = $fontList[$i]->font; 
						$delString = "Are you sure you want to delete \'".$curFont."\'? This will also delete any CSS associations.";
						
						if ($i & 1)
							echo '<tr class="rowOdd">';
						else 
							echo '<tr class="rowEven">';
							 
						echo '
								<td>'.$curName.'</td>
								<td>'.$curFont.'</td>
								<td>
	
								<form action="?page=ttfgen.php&action=deleteFont" method="post" id="ttfgen_delete" name="ttfgen_delete">  
									<input type="hidden" name="fontID" id="fontID" value="'.$curID.'" />
									<input name="Delete" type="submit" value="Delete" onclick="return confirmDelete(\''.$delString.'\')"/>
								</form> 
								
								</td>
							</tr>
						';
					}
					
				// close table
				echo '</table>'; 
					
				}
			?>
		</div>
	
	<div class="colorWrapper">

		<h3>Add a New Font</h3>
		
		<?php 
	
		$max_upload_size = 1000000; 
		$max_upload_size_text = '';					
			
		?>
			<form enctype="multipart/form-data" action="?page=ttfgen.php&action=add&method=upload" method="post" id="ttfgen_add" name="ttfgen_add" class="form-table"> 
	            <input type="hidden" name="MAX_FILE_SIZE" value="<?php echo $max_upload_size; ?>" />
	            <table class="optiontable niceblue" cellpadding="0" cellspacing="0"> 
	                <tr>
	                    <th scope="row" style="width:100px"><strong><?php _e('Font Name',"wp-ttfgen"); ?></strong></th> 
	                    <td>
	                        <input type="text" style="width:320px;" class="cleardefault" name="fontName" id="fontName" maxlength="200" />												
	                    </td> 
	                </tr>
					<tr valign="top">
							<th scope="row" style="width:100px"><strong><?php _e('Select a font',"wp-ttfgen"); ?></strong></th> 
							<td><input type="file" name="uploadedFile" style="width:320px;" /><br />
								<span class="setting-description">
									<?php _e('Max. filesize',"wp-ttfgen"); echo $max_upload_size_text; ?> = <?php echo $max_upload_size; ?> <?php _e('bytes',"wp-ttfgen"); ?>.
									<br /> <?php _e('If a font with the same name already exists it will be renamed automatically.',"wp-ttfgen"); ?>
								</span>
							</td>
	                </tr> 
					<tr>
						<td>
						<input type="submit" value="Add Font" name="submit" /></td>
					</tr>
	
	            </table>
			 
			</form>

	</div>
	
	<div class="sectionWrapper">
		
		<h3>CSS Font Assignments</h3>
			
            <table id="cssAddTable" class="optiontable niceblue" cellpadding="0" cellspacing="0"> 

			</table>
			
				
			<table id="cssListTable" cellpadding="0" cellspacing="0"> 
			  <tr>
                    <th><strong><?php _e('CSS Type',"wp-ttfgen"); ?></strong></th> 
					<th><strong><?php _e('CSS Name',"wp-ttfgen"); ?></strong></th> 
					<th><strong><?php _e('Font to Use',"wp-ttfgen"); ?></strong></th>
					<th><strong><?php _e('Action',"wp-ttfgen"); ?></strong></th>
                </tr>

				
				<tr>
					<form action="?page=ttfgen.php&action=assignCSS" method="post" id="ttfgen_assign" name="ttfgen_assign" class="form-table"> 
						<td>
							<input type="radio" name="cssType" value="ID" /> ID
							<input type="radio" name="cssType" value="Class" /> Class
						</td>
						<td><input type="text" name="cssName" id="cssName" maxlength="100" /></td>
						<td>
							<select id="fontName" name="fontName">		
								<?php 
									for ($i=0; $i < count($fontList); $i++) {
										echo '<option value="'.$fontList[$i]->name.'">'.$fontList[$i]->name.'</option>';
									}
								?>
							</select>
							
						</td>
						<td><input type="submit" value="Assign Font" name="Assign Font"></td>	
					</form>		
				</tr>		
				
				<?php  
					$cssList = getCSS(); 
					
					if (count($cssList) > 0)
					{			
						// Display the existing CSS List						
						for ($i=0; $i < count($cssList); $i++) 
						{
							$cssID = $cssList[$i]->id;
							$type = $cssList[$i]->css_type; 
							$name = $cssList[$i]->css_name; 
							$fontID = $cssList[$i]->css_font_id; 
							$fontName = getFontName($fontID); // get the the font name from the ID
							$cssDelString = "Are you sure you want to delete the CSS assignment for \'".$name."\'?";

							if ($i & 1)
								echo '<tr class="rowEven">';
							else 
								echo '<tr class="rowOdd">';
													
							echo '
									<td>'.$type.'</td>
									<td>'.$name.'</td>
									<td>'.$fontName.'</td>
									<td >
										<form action="?page=ttfgen.php&action=deleteCSS" method="post" id="ttfgen_delete_css" name="ttfgen_delete_css">  
											<input type="hidden" name="cssID" id="cssID" value="'.$cssID.'" />
											<input name="Delete" type="submit" value="Delete" onclick="return confirmDelete(\''.$cssDelString.'\')"/>
										</form>
									</td> 
								</tr>
							';
						}
						
					}
				?>
            </table>	
					
		</div>

	<div class="footerSpacer" style="padding-top: 20px; clear:both;"></div>
		
</div>
	

		
	
<?php
	}

// admin Options Page End


/**
 * 
 * PAGE LOAD FUNCTION 
 * This function will take the parameters specified in the admin panel 
 * and load the jQuery options within the wp_footer() section of the site
 * 
 */

function ttfLoad() 
{
	global $wpdb, $ttfOptions;

	$fontList = getFonts(); 
	$cssList = getCSS();
	
	// if no fonts are found or no CSS assignments are set, end the program. 
	if ($fontList == null || $cssList == null) 
	{
		die();
	}
	else 
	{
		echo "\n\n<!-- Start TTFGen Code -->\n\n"; 
				
		// if useJquery option is true, load jQuery
		if ($ttfOptions['useJquery'])
			echo "<script type='text/javascript' src='".TTF_PATH_SHORT."scripts/jquery-min.js'></script>\n"; 

		// if useMeta option is true, load jQuery metadata plugin
		if ($ttfOptions['useMeta'])
			echo "<script type='text/javascript' src='".TTF_PATH_SHORT."scripts/jquery.metadata.js'></script>\n"; 		
		
		// load the jQuery TTFGen file 
		echo "<script type='text/javascript' src='".TTF_PATH_SHORT."scripts/jquery.ttfgen.js'></script>\n\n"; 

		// Begin main script load
		echo "<script type='text/javascript'>\n";
		echo "   $(document).ready(function() {\n";  
		
		// Begin the cycle through the cssList to create the calls to ttfgen
		for ($i=0; $i < count($cssList); $i++) 
		{
			$type = $cssList[$i]->css_type; 
			$name = $cssList[$i]->css_name; 
			$fontID = $cssList[$i]->css_font_id; 
			$fontName = getFontName($fontID); // get the the font name from the ID 
			
			
			// class or ID?
			if ($type == "Class")
				$ident = "."; 
			else if ($type == "ID")
				$ident = "#";
				 
			// echo the final jQuery string 	
			echo "      $('".$ident.$name."').ttfgen( { font: '".$fontName."', asBackground: true } );\n"; 

		}
		
		// end script 
		echo "   });\n"; 
		echo "</script>\n\n"; 	 
		
		echo "<!-- End TTFGen Code -->\n\n"; 
		
	}
}

// Start this plugin once all other plugins are fully loaded
add_action('admin_menu', 'ttfAdminMenu'); 
 
// load TTFGen in the header or footer
if ($ttfOptions['loadLocation'] == "header")
	add_action('wp_header', 'ttfLoad');
else 
	add_action('wp_footer', 'ttfLoad');



?>